/*
C*GRCLPL -- clip line against clipping rectangle
C+
      SUBROUTINE GRCLPL (X0,Y0,X1,Y1,VIS)
C
C GRPCKG (internal routine): Change the end-points of the line (X0,Y0)
C (X1,Y1) to clip the line at the window boundary.  The algorithm is
C that of Cohen and Sutherland (ref: Newman & Sproull).
C
C Arguments:
C
C X0, Y0 (input/output, real): device coordinates of starting point
C       of line.
C X1, Y1 (input/output, real): device coordinates of end point of line.
C VIS (output, logical): .TRUE. if line lies wholly or partially
C       within the clipping rectangle; .FALSE. if it lies entirely
C       outside the rectangle.
C--
C 13-Jul-1984 - [TJP].
C 20-Jun-1985 - [TJP] - revise clipping algorithm.
C 28-Jun-1991 - [TJP] - use IAND().
C 12-Jun-1992 - [TJP] - clip exactly on the boundary.
C
C Caution: IAND is a non-standard intrinsic function to do bitwise AND
C of two integers. If it is not supported by your Fortran compiler, you
C will need to modify this routine or supply an IAND function.
C-----------------------------------------------------------------------
*/

#include "pgplot.h"
#define TRUE 1
#define FALSE 0

void GRCLPL(float *X0, float *Y0, float *X1, float *Y1, int *VIS) {
	int C0,C1,C;
	float XMIN, XMAX, YMIN, YMAX;
	float X, Y;
	int cgrcide = grcm00_.grcide - 1;

	XMIN = grcm00_.grxmin[cgrcide];
	YMIN = grcm00_.grymin[cgrcide];
	XMAX = grcm00_.grxmax[cgrcide];
	YMAX = grcm00_.grymax[cgrcide];
	
	GRCLIP(X0, Y0, &XMIN, &XMAX, &YMIN, &YMAX, &C0);
	GRCLIP(X1, Y1, &XMIN, &XMAX, &YMIN, &YMAX, &C1);
	
	while (C0 != 0 || C1 != 0) {
		if ((C0 & C1) != 0) {
/*             ! line is invisible */
			*VIS = FALSE;
			return;
		}
		C = C0;
		if (C == 0) C = C1;
		if ((C & 1) != 0) {
/*             ! crosses XMIN */
			Y = *Y0 + (*Y1 - *Y0) * (XMIN - *X0) / (*X1 - *X0);
			X = XMIN;
		} else if ((C & 2) != 0) {
/*             ! crosses XMAX */
			Y = *Y0 + (*Y1 - *Y0) * (XMAX - *X0) / (*X1 - *X0);
			X = XMAX;
		} else if ((C & 4) != 0) {
/*             ! crosses YMIN */
			X = *X0 + (*X1 - *X0) * (YMIN - *Y0) / (*Y1 - *Y0);
			Y = YMIN;
		} else if ((C & 8) != 0) {
/*             ! crosses YMAX */
			X = *X0 + (*X1 - *X0) * (YMAX - *Y0) / (*Y1 - *Y0);
			Y = YMAX;
		}
		if (C ==C0) {
			*X0 = X;
			*Y0 = Y;
			GRCLIP(&X, &Y, &XMIN, &XMAX, &YMIN, &YMAX, &C0);
		} else {
			*X1 = X;
			*Y1 = Y;
			GRCLIP(&X, &Y, &XMIN, &XMAX, &YMIN, &YMAX, &C1);
		}
	}
	*VIS = TRUE;
}
